package eu.jonahbauer.wizard.client.libgdx.actions.overlay;

import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.Group;
import eu.jonahbauer.wizard.client.libgdx.WizardGame;
import eu.jonahbauer.wizard.client.libgdx.actors.PadOfTruth;
import eu.jonahbauer.wizard.client.libgdx.screens.GameScreen;
import eu.jonahbauer.wizard.client.libgdx.util.AnimationTimings;
import org.jetbrains.annotations.NotNull;

import java.util.Objects;

import static com.badlogic.gdx.scenes.scene2d.actions.Actions.*;
import static eu.jonahbauer.wizard.client.libgdx.actions.MyActions.changeParent;

public class ScoreOverlay extends Overlay {
    private final PadOfTruth padOfTruth;

    public ScoreOverlay(@NotNull GameScreen gameScreen) {
        super(gameScreen, Long.MAX_VALUE);
        padOfTruth = Objects.requireNonNull(gameScreen.getPadOfTruth());
    }

    @Override
    protected Actor createContent() {
        return padOfTruth;
    }

    @Override
    protected void show(Group parent) {
        super.show(parent);

        var root = getRoot();
        root.addAction(sequence(
                run(() -> padOfTruth.setEnabled(false)),
                parallel(
                        targeting(padOfTruth, scaleTo(1, 1, AnimationTimings.OVERLAY_SHARED_ELEMENT)),
                        targeting(padOfTruth, moveTo((WizardGame.WIDTH - padOfTruth.getWidth()) / 2, (WizardGame.HEIGHT - padOfTruth.getHeight()) / 2, AnimationTimings.OVERLAY_SHARED_ELEMENT))
                ),
                delay(AnimationTimings.OVERLAY_HOLD),
                parallel(
                        targeting(padOfTruth, scaleTo(PadOfTruth.COLLAPSED_SCALE, PadOfTruth.COLLAPSED_SCALE, AnimationTimings.OVERLAY_SHARED_ELEMENT)),
                        targeting(padOfTruth, moveTo(WizardGame.WIDTH - 10 - PadOfTruth.EXTENDED_WIDTH, 10, AnimationTimings.OVERLAY_SHARED_ELEMENT))
                ),
                targeting(padOfTruth, changeParent(screen.getContentRoot())),
                run(() -> padOfTruth.setEnabled(true)),
                run(this::close)
        ));
    }
}
