package eu.jonahbauer.wizard.client.libgdx.actions.overlay;

import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.Group;
import com.badlogic.gdx.scenes.scene2d.actions.ParallelAction;
import com.badlogic.gdx.scenes.scene2d.ui.HorizontalGroup;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.badlogic.gdx.scenes.scene2d.ui.VerticalGroup;
import eu.jonahbauer.wizard.client.libgdx.util.AnimationTimings;
import eu.jonahbauer.wizard.client.libgdx.actions.MyActions;
import eu.jonahbauer.wizard.client.libgdx.actors.CardActor;
import eu.jonahbauer.wizard.client.libgdx.screens.GameScreen;
import eu.jonahbauer.wizard.common.model.Card;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.Map;
import java.util.Objects;

import static com.badlogic.gdx.scenes.scene2d.actions.Actions.*;
import static eu.jonahbauer.wizard.client.libgdx.actions.MyActions.*;

public class TrumpOverlay extends Overlay {
    @SuppressWarnings("RedundantTypeArguments")
    private static final Map<Card, Card.Suit> DEFAULT_SUITES = Map.<Card, Card.Suit>ofEntries(
            Map.entry(Card.BLUE_1, Card.Suit.BLUE),
            Map.entry(Card.BLUE_2, Card.Suit.BLUE),
            Map.entry(Card.BLUE_3, Card.Suit.BLUE),
            Map.entry(Card.BLUE_4, Card.Suit.BLUE),
            Map.entry(Card.BLUE_5, Card.Suit.BLUE),
            Map.entry(Card.BLUE_6, Card.Suit.BLUE),
            Map.entry(Card.BLUE_7, Card.Suit.BLUE),
            Map.entry(Card.BLUE_8, Card.Suit.BLUE),
            Map.entry(Card.BLUE_9, Card.Suit.BLUE),
            Map.entry(Card.BLUE_10, Card.Suit.BLUE),
            Map.entry(Card.BLUE_11, Card.Suit.BLUE),
            Map.entry(Card.BLUE_12, Card.Suit.BLUE),
            Map.entry(Card.BLUE_13, Card.Suit.BLUE),
            Map.entry(Card.RED_1, Card.Suit.RED),
            Map.entry(Card.RED_2, Card.Suit.RED),
            Map.entry(Card.RED_3, Card.Suit.RED),
            Map.entry(Card.RED_4, Card.Suit.RED),
            Map.entry(Card.RED_5, Card.Suit.RED),
            Map.entry(Card.RED_6, Card.Suit.RED),
            Map.entry(Card.RED_7, Card.Suit.RED),
            Map.entry(Card.RED_8, Card.Suit.RED),
            Map.entry(Card.RED_9, Card.Suit.RED),
            Map.entry(Card.RED_10, Card.Suit.RED),
            Map.entry(Card.RED_11, Card.Suit.RED),
            Map.entry(Card.RED_12, Card.Suit.RED),
            Map.entry(Card.RED_13, Card.Suit.RED),
            Map.entry(Card.GREEN_1, Card.Suit.GREEN),
            Map.entry(Card.GREEN_2, Card.Suit.GREEN),
            Map.entry(Card.GREEN_3, Card.Suit.GREEN),
            Map.entry(Card.GREEN_4, Card.Suit.GREEN),
            Map.entry(Card.GREEN_5, Card.Suit.GREEN),
            Map.entry(Card.GREEN_6, Card.Suit.GREEN),
            Map.entry(Card.GREEN_7, Card.Suit.GREEN),
            Map.entry(Card.GREEN_8, Card.Suit.GREEN),
            Map.entry(Card.GREEN_9, Card.Suit.GREEN),
            Map.entry(Card.GREEN_10, Card.Suit.GREEN),
            Map.entry(Card.GREEN_11, Card.Suit.GREEN),
            Map.entry(Card.GREEN_12, Card.Suit.GREEN),
            Map.entry(Card.GREEN_13, Card.Suit.GREEN),
            Map.entry(Card.YELLOW_1, Card.Suit.YELLOW),
            Map.entry(Card.YELLOW_2, Card.Suit.YELLOW),
            Map.entry(Card.YELLOW_3, Card.Suit.YELLOW),
            Map.entry(Card.YELLOW_4, Card.Suit.YELLOW),
            Map.entry(Card.YELLOW_5, Card.Suit.YELLOW),
            Map.entry(Card.YELLOW_6, Card.Suit.YELLOW),
            Map.entry(Card.YELLOW_7, Card.Suit.YELLOW),
            Map.entry(Card.YELLOW_8, Card.Suit.YELLOW),
            Map.entry(Card.YELLOW_9, Card.Suit.YELLOW),
            Map.entry(Card.YELLOW_10, Card.Suit.YELLOW),
            Map.entry(Card.YELLOW_11, Card.Suit.YELLOW),
            Map.entry(Card.YELLOW_12, Card.Suit.YELLOW),
            Map.entry(Card.YELLOW_13, Card.Suit.YELLOW),
            Map.entry(Card.RED_JESTER, Card.Suit.NONE),
            Map.entry(Card.GREEN_JESTER, Card.Suit.NONE),
            Map.entry(Card.BLUE_JESTER, Card.Suit.NONE),
            Map.entry(Card.YELLOW_JESTER, Card.Suit.NONE),
            Map.entry(Card.BOMB, Card.Suit.NONE),
            Map.entry(Card.FAIRY, Card.Suit.NONE)
    );

    private final @Nullable String player;
    private final @Nullable Card card;
    private final @Nullable Card.Suit suit;

    private final @NotNull CardActor trumpCardActor;
    private final @NotNull CardActor trumpSuitActor;

    private boolean animateCard = true;

    public TrumpOverlay(@NotNull GameScreen gameScreen, @Nullable String player, @Nullable Card card, @Nullable Card.Suit suit) {
        super(gameScreen, Long.MAX_VALUE);
        this.player = player;
        this.card = card;
        this.suit = suit;
        this.trumpCardActor = Objects.requireNonNull(gameScreen.getTrumpCardActor());
        this.trumpSuitActor = Objects.requireNonNull(gameScreen.getTrumpSuitActor());
    }

    @Override
    public Actor createContent() {
        var root = new VerticalGroup().columnCenter().space(10f);

        String text;

        if (player == null) {
            text = messages.get(suit != null ? switch (suit) {
                case YELLOW -> "game.overlay.trump.yellow";
                case GREEN -> "game.overlay.trump.green";
                case BLUE -> "game.overlay.trump.blue";
                case RED -> "game.overlay.trump.red";
                default -> "game.overlay.trump.none";
            } : "game.overlay.trump.unknown");
        } else {
            text = messages.format(suit != null ? switch (suit) {
                case YELLOW -> "game.overlay.trump.yellow.player";
                case GREEN -> "game.overlay.trump.green.player";
                case BLUE -> "game.overlay.trump.blue.player";
                case RED -> "game.overlay.trump.red.player";
                default -> "game.overlay.trump.none.player";
            } : "game.overlay.trump.unknown.player", player);
        }

        var label = new Label(text, skin);
        label.getStyle().font.getData().markupEnabled = true;
        label.setFontScale(1.5f);
        root.addActor(label);

        var cardGroup = new HorizontalGroup().space(20);
        root.addActor(cardGroup);

        if (trumpCardActor.getParent() != null && trumpCardActor.getCard() == card && suit != null) {
            // if card actor is already correct then dont change it
            animateCard = false;
        } else {
            cardGroup.addActor(trumpCardActor);
            trumpCardActor.setCard(card != null ? card : Card.HIDDEN);
        }

        if (suit != null && suit != DEFAULT_SUITES.get(card)) {
            trumpSuitActor.setRotation(0);
            trumpSuitActor.setOrigin(0, 0);
            cardGroup.addActor(trumpSuitActor);
            trumpSuitActor.setCard(suit);
        } else {
            trumpSuitActor.remove();
        }

        return root;
    }

    @Override
    public void show(Group parent) {
        super.show(parent);

        ParallelAction cardAnimation = parallel();
        if (animateCard) {
            cardAnimation.addAction(sequence(
                    targeting(trumpCardActor, removeActorSilently()),
                    targeting(trumpSuitActor, changeParent(screen.getContentRoot())),
                    targeting(trumpCardActor, moveTo(10, 10, AnimationTimings.OVERLAY_SHARED_ELEMENT))
            ));
        }

        if (suit != null && suit != DEFAULT_SUITES.get(card)) {
            cardAnimation.addAction(sequence(
                    targeting(trumpSuitActor, removeActorSilently()),
                    targeting(trumpSuitActor, changeParent(screen.getContentRoot())),
                    run(trumpCardActor::toFront),
                    parallel(
                            targeting(trumpSuitActor, rotateTo(-90, AnimationTimings.OVERLAY_SHARED_ELEMENT)),
                            targeting(trumpSuitActor,
                                      moveTo(10, 10 + (trumpSuitActor.getHeight() + trumpSuitActor.getWidth()) / 2, AnimationTimings.OVERLAY_SHARED_ELEMENT)
                            )
                    )
            ));
        }

        var root = getRoot();
        root.addAction(sequence(
                delay(AnimationTimings.OVERLAY_HOLD),
                parallel(
                        cardAnimation,
                        run(this::close)
                )
        ));
    }

    @Override
    protected void onClosing() {
        var actions = getRoot().getActions();
        if (actions.size > 0) MyActions.finish(actions.get(0));
        onClosed();
    }
}
